/** @file  uapcmd.h
 *
 *  @brief This file contains declaration referring to
 *  functions defined in uapcmd.c
 * 
 * Copyright (C) 2008-2009, Marvell International Ltd. 
 *
 * This software file (the "File") is distributed by Marvell International 
 * Ltd. under the terms of the GNU General Public License Version 2, June 1991 
 * (the "License").  You may use, redistribute and/or modify this File in 
 * accordance with the terms and conditions of the License, a copy of which 
 * is available along with the File in the gpl.txt file or by writing to 
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 
 * 02111-1307 or on the worldwide web at http://www.gnu.org/licenses/gpl.txt.
 *
 * THE FILE IS DISTRIBUTED AS-IS, WITHOUT WARRANTY OF ANY KIND, AND THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE 
 * ARE EXPRESSLY DISCLAIMED.  The License provides additional details about 
 * this warranty disclaimer.
 *
 */
/************************************************************************
Change log:
    03/01/08: Initial creation
************************************************************************/

#ifndef _UAPCMD_H
#define _UAPCMD_H

/** Function Prototype Declaration */
void apcmd_sys_cfg_ap_mac_address(int argc, char *argv[]);
void apcmd_sys_cfg_ssid(int argc, char *argv[]);
void apcmd_sys_cfg_beacon_period(int argc, char *argv[]);
void apcmd_sys_cfg_dtim_period(int argc, char *argv[]);
void apcmd_sys_cfg_channel(int argc, char *argv[]);
void apcmd_sys_cfg_scan_channels(int argc, char *argv[]);
void apcmd_sys_cfg_rates(int argc, char *argv[]);
void apcmd_sys_cfg_rates_ext(int argc, char *argv[]);
void apcmd_sys_cfg_tx_power(int argc, char *argv[]);
void apcmd_sys_cfg_bcast_ssid_ctl(int argc, char *argv[]);
void apcmd_sys_cfg_preamble_ctl(int argc, char *argv[]);
void apcmd_sys_cfg_antenna_ctl(int argc, char *argv[]);
void apcmd_sys_cfg_rts_threshold(int argc, char *argv[]);
void apcmd_sys_cfg_frag_threshold(int argc, char *argv[]);
void apcmd_sys_cfg_radio_ctl(int argc, char *argv[]);
void apcmd_sys_cfg_rsn_replay_prot(int argc, char *argv[]);
void apcmd_sys_cfg_tx_data_rate(int argc, char *argv[]);
void apcmd_sys_cfg_mcbc_data_rate(int argc, char *argv[]);
void apcmd_sys_cfg_pkt_fwd_ctl(int argc, char *argv[]);
void apcmd_sys_cfg_sta_ageout_timer(int argc, char *argv[]);
void apcmd_sys_cfg_auth(int argc, char *argv[]);
void apcmd_sys_cfg_protocol(int argc, char *argv[]);
void apcmd_sys_cfg_wep_key(int argc, char *argv[]);
void apcmd_sys_cfg_custom_ie(int argc, char *argv[]);
void apcmd_sys_cfg_cipher(int argc, char *argv[]);
void apcmd_sys_cfg_wpa_passphrase(int argc, char *argv[]);
void apcmd_sys_cfg_group_rekey_timer(int argc, char *argv[]);
void apcmd_coex_config_profile(int argc, char *argv[]);
void apcmd_sta_filter_table(int argc, char *argv[]);
void apcmd_sys_cfg_max_sta_num(int argc, char *argv[]);
void apcmd_sys_cfg_retry_limit(int argc, char *argv[]);
void apcmd_cfg_data(int argc, char *argv[]);
#endif /* _UAP_H */
